# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.auth.decorators import login_required, user_passes_test
from django.shortcuts import render_to_response, redirect
from django.http import Http404
from django.template import RequestContext
from django.core.files import File
from django.core.urlresolvers import reverse
from mongoengine.connection import get_db
from mongoengine.queryset import Q
from forms import UploadPictureForm
from helper import UploadPicture, tag_frequencies
from models import Post
from tags.models import Tag
from utils.helper import Paginator
from gridfs import GridFS

# TODO: Add a way for adding default tags
@login_required
def upload(request):
    if request.method == 'POST':
        form = UploadPictureForm(request.POST, request.FILES)
        if form.is_valid():
            for request_file in request.FILES.getlist('files'):
                picture = UploadPicture()
                picture.readFile(request_file)
                post = Post()

                # We need the ID, so we need to save the post first
                post.save(validate=False)
                picture.renameFiles(str(post.pk))

                fs = GridFS(get_db())
                p_id= fs.put(filename=picture.path, data=File(open('/tmp/' + picture.path, 'r')), contentType=picture.contentType)
                sp_id = fs.put(filename=picture.smallPath, data=File(open('/tmp/' + picture.smallPath, 'r')), contentType="image/jpeg")
                tp_id = fs.put(filename=picture.thumbPath, data=File(open('/tmp/' + picture.thumbPath, 'r')), contentType="image/jpeg")
                post.picture = fs.get(p_id).name
                post.small_picture = fs.get(sp_id).name
                post.thumb_picture = fs.get(tp_id).name
                post.picture_height = picture.height
                post.picture_width = picture.width
                post.small_height = picture.small_height
                post.small_width = picture.small_width
                post.thumb_height = picture.thumb_height
                post.thumb_width = picture.thumb_width
                post.type = picture.type
                tag = Tag.objects(name='tagme').first()
                if tag is None:
                    tag = Tag(name='tagme', count=1)
                    tag.save()
                else:
                    Tag.objects(name='tagme').update_one(inc__count=1)
                post.tags.append('tagme')
                post.uploader = request.user
                post.save()

                request_file.close()
                picture.delete_pictures()

            return redirect('posts.views.list')
    else:
        form = UploadPictureForm()
    # We need the RequestContext for the CSRF protection
    return render_to_response('posts/upload.html', {'form': form}, context_instance=RequestContext(request))

def details(request, id):
    post = Post.objects().exclude('thumb_picture', 'thumb_width', 'thumb_height').with_id(id)
    if post is None:
        raise Http404
    tags = Tag.objects(name__in=post.tags).order_by('name')
    return render_to_response('posts/details.html', {'media_url': settings.MEDIA_URL,
                                                    'post': post,
                                                    'tags': tags,},
                                                    context_instance=RequestContext(request))

@user_passes_test(lambda u: u.is_staff)
def delete(request, id):
    post = Post.objects().with_id(id)
    if post is None:
        raise Http404
    for tag in Tag.objects(name__in=post.tags):
        if tag.count > 0:
            Tag.objects(id=tag.pk).update_one(dec__count=1)

    fs = GridFS(get_db())
    if fs.exists(filename=post.picture):
        fs.delete(fs.get_last_version(post.picture)._id)
    if fs.exists(filename=post.small_picture):
        fs.delete(fs.get_last_version(post.small_picture)._id)
    if fs.exists(filename=post.thumb_picture):
            fs.delete(fs.get_last_version(post.thumb_picture)._id)

    post.delete()
    return redirect('posts.views.list')


# TODO: Support AJAX
def add_tag(request, id):
    if request.GET.get('tag'):
        tag_name = request.GET['tag']
        post = Post.objects().with_id(id)
        if post is None:
            raise Http404
        if not tag_name in post.tags:
            post.tags.append(tag_name)
            post.save()
            tag = Tag.objects(name=tag_name).first()
            if tag is None:
                tag = Tag(name=tag_name, count=1)
                tag.save()
            else:
                Tag.objects(id=tag.pk).update_one(inc__count=1)
    return redirect('posts.views.details', id)

def list(request):
    items_per_page = 21
    page = 1
    query = None
    try:
        if request.GET.get('page'):
            page = int(request.GET['page'])
        else:
            page = 1
        # TODO: More queries and variable picture count on one page
        if request.GET.get('tags'):
                query = Q(tags__all=request.GET['tags'].split())
                tags_html = 'tags=' + request.GET['tags'] + '&'
        else:
            tags_html = ''
    except:
        tags_html = ''

    count = Post.objects(query).only().count()
    paginator = Paginator(count, items_per_page)
    paginator.set_page(page)
    paginator.html_query = reverse('posts.views.list') + '?' + tags_html +  'page='

    skip = (paginator.current_page - 1) * items_per_page
    posts = Post.objects(query)\
                .only('tags','thumb_picture', 'thumb_height', 'thumb_width' ,'picture', 'picture_height', 'picture_width')\
                [skip:skip+items_per_page]

    if count != 0:
        top_tags = Tag.objects(name__in=tag_frequencies(posts)).order_by('name')
    else:
        top_tags = ()

    return render_to_response('posts/list.html', {'posts': posts,
                                                  'tags': top_tags,
                                                  'media_url': settings.MEDIA_URL,
                                                  'paginator': paginator,},
                                                  context_instance=RequestContext(request) )
